@@ -4,7 +4,7 @@
 
 ;; Emacs Lisp Archive Entry
 ;; Filename: planner-authz.el
-;; Version: $Revision: 1.20 $
+;; Version: $Revision: 1.26 $
 ;; Keywords: hypermedia
 ;; Author: Andrew J. Korty <ajk@iu.edu>
 ;; Maintainer: Andrew J. Korty <ajk@iu.edu>
@@ -100,13 +100,10 @@
 
 ;;; Code
 
-;; $Id: planner-authz.el,v 1.20 2004/06/24 20:41:49 ajk Exp $
+;; $Id: planner-authz.el,v 1.26 2005/03/13 14:26:36 ajk Exp $
 
 (require 'planner)
 
-(eval-when-compile
-  (require 'cl))
-
 ;; Customization options
 
 ;;; Code:
@@ -116,6 +113,31 @@ published pages to specified users."
   :group 'planner
   :prefix "planner-authz")
 
+(defcustom planner-authz-appt-alt nil
+  "If non-nil, show `planner-appt' appointments to users not
+authorized to see them, but replace the text of the appointment with
+the contents of this variable.  If nil, don't show any part of an
+appointment to an unauthorized user.
+
+For example, if this variable is set to \"Private appointment\" and
+some hypothetical user is not authorized for the SecretStuff page, an
+appointment that was entered as
+
+ #A1  _ @10:00 12:00 Secret meeting (SecretStuff)
+
+would appear to our unauthorized user as
+
+ #A1  _ @10:00 12:00 Private appointment"
+  :group 'planner
+  :type '(choice (string :tag "Replacement text")
+                 (const :tag "Disable" nil)))
+
+(defcustom planner-authz-appt-regexp
+  (concat
+   "[!@]?\\s-*[0-9]?[0-9]:[0-5][0-9]\\(?:\\s-*[ap]m?\\)?"
+   "\\(?:\\s-*-?\\s-*[0-9]?[0-9]:[0-5][0-9]\\(?:\\s-*[ap]m?\\)?\\)?\\s-")
+  "Regexp that matches a `planner-appt' start and end time specification.")
+
 (defcustom planner-authz-mason-component-contents
   "<%once>
 sub authz {
@@ -124,14 +146,17 @@ sub authz {
         foreach (@_) { return 1 if $r_user eq $_ }
         return 0;
 }
-</%once>
-<%method content>
-<%args>@users</%args>
-% if (authz @users) {
-        <% $m->content %>
-% }
-</%method>
-<%method page>
+  </%once>
+  <%method content>
+  <%args>
+  $alt    => undef
+  @users
+  </%args>
+  % if (authz @users) {
+  <% $m->content %>\+% } elsif ($alt) {
+  <% $alt %>\ % }
+  </%method>
+  <%method page>
 <%args>@users</%args>
 <%perl>
 unless (authz @users) {
@@ -190,8 +215,8 @@ emacs-wiki rules."
 
     ;; Move Masonry to outside of list item tags.
 
-    ["\\(<li>\\)\\(<&[^&]*&>\\)" 0 "\\2\\1"]
-    ["\\(</&>\\)\\(\\s-*\\)\\(</li>\\)" 0 "\\2\\3\\1"]
+    ["\\(<li>\\)\\(<&[^&]*&>\\)\\(.*\\)\\(</&>\\)\\(\\s-*\\)\\(</li>\\)" 0
+     "\\2\\1\\3\\5\\6\\4"]
 
     ;; Remove <p> tags from inserted Masonry.
 
@@ -210,6 +235,9 @@ emacs-wiki rules."
 
 ;; Non-customizable variables
 
+(defvar planner-authz-initialized-p nil
+  "Non-nil if planner-authz has run its initialization code.")
+
 (defvar planner-authz-pages nil
   "Alist of planner pages and users authorized to view them.
 The list of users is separated by spaces.  This variable is
@@ -229,17 +257,28 @@ emacs-wiki to insert <code> tags.")
 (defvar planner-authz-mason-content-tag-begin
   "<&: authz.mas:content, 'users', [qw(%s)] &>"
   "Mason component call that restricts content to a list of users.
-If this variable contains \"<&:\", it will be replaced by \"<&|\"
-at the end of the publishing process to avoid being marked up as
-a table by emacs-wiki.  Don't be tempted to use the => Perl
-operator; the = could cause emacs-wiki to insert <code> tags.")
+The %s is replaced by a space-separated list of these users.  If this
+variable contains \"<&:\", it will be replaced by \"<&|\" at the end
+of the publishing process to avoid being marked up as a table by
+emacs-wiki.  Don't be tempted to use the => Perl operator; the = could
+cause emacs-wiki to insert <code> tags.")
+(defvar planner-authz-mason-content-tag-alt-begin
+  "<&: authz.mas:content, 'users', [qw(%s)], 'alt', '%s' &>"
+  "Mason component call that restricts content to a list of users and
+shows alternate content to other users.  The first %s is replaced by a
+space-separated list of these users.  The second %s is replaced by
+alternate text to show to users who are not authorized.  If this
+variable contains \"<&:\", it will be replaced by \"<&|\" at the end
+of the publishing process to avoid being marked up as a table by
+emacs-wiki.  Don't be tempted to use the => Perl operator; the = could
+cause emacs-wiki to insert <code> tags.")
 (defvar planner-authz-mason-content-tag-end "</&>"
   "Mason tag to insert at the end of content to be restricted.")
 
 (defvar planner-authz-rule '("authz" t t nil planner-authz-tag)
   "Rule to add to `emacs-wiki-dangerous-tags' for <authz> tag support.")
 
-(defvar planner-authz-version "$Revision: 1.20 $"
+(defvar planner-authz-version "$Revision: 1.26 $"
   "Version of of planner-authz.el.")
 
 ;; Cleanup function to run in buffer after markup
@@ -379,13 +418,10 @@ component restricts access to users specified by <authz> and
   "Function to be run from `planner-mode-hook'.  Useful for
 tweaking emacs-wiki variables that planner overrides."
 
-  ;; Add our markup rules, preserving those of planner and emacs-wiki
+  (unless planner-authz-initialized-p   ; run this code only once
+
+    ;; Add our markup rules, preserving those of planner and emacs-wiki
 
-  (let ((base-markup (copy-sequence emacs-wiki-publishing-markup)))
-    (dolist (elt planner-authz-publishing-markup-first)
-      (setq base-markup (delete elt base-markup)))
-    (dolist (elt planner-authz-publishing-markup-last)
-      (setq base-markup (delete elt base-markup)))
     (setq planner-custom-variables
           (assq-delete-all 'emacs-wiki-publishing-markup
                            planner-custom-variables))
@@ -394,13 +430,15 @@ tweaking emacs-wiki variables that planner overrides."
      `(emacs-wiki-publishing-markup
        . ,(append
            planner-authz-publishing-markup-first
-           base-markup
-           planner-authz-publishing-markup-last))))
+           emacs-wiki-publishing-markup
+           planner-authz-publishing-markup-last)))
 
-  ;; Make all our tweaks take effect
+    ;; Make our tweak take effect
 
-  (planner-option-customized 'planner-custom-variables
-                             planner-custom-variables))
+    (planner-option-customized 'planner-custom-variables
+                               planner-custom-variables))
+
+  (setq planner-authz-initialized-p t))
 
 (defun planner-authz-initial-directive-preprocess ()
   "Build `planner-authz-pages' from all pages containing #authz.
@@ -408,12 +446,13 @@ Pages that are subsequently published get removed from this list;
 all others are forcibly republished."
   (setq planner-authz-pages nil
         planner-authz-pages-to-republish nil)
-  (dolist (elt (planner-file-alist))
-    (let ((page (car elt)))
+  (let ((pages (planner-file-alist)))
+    (while pages
       (with-temp-buffer
-        (insert-file-contents-literally (cdr elt))
+        (insert-file-contents-literally (cdar pages))
         (while (re-search-forward "^#authz\\s-+\\(.+\\)\n+" nil t)
-          (push `(,page . ,(match-string 1)) planner-authz-pages))))))
+          (push `(,(caar pages) . ,(match-string 1)) planner-authz-pages)))
+      (setq pages (cdr pages)))))
 
 (defun planner-authz-markup-note ()
   "Restrict notes linked to a restricted page.
@@ -451,7 +490,8 @@ as it appears there."
        linked-users
        (lambda ()
          (save-match-data
-           (if (re-search-forward "^\\*\\{1,3\\}\\s-+" nil 0 2)
+           (if (re-search-forward
+                "^\\(?:\\.#[0-9][0-9]*\\|\\*\\{1,3\\}\\)\\s-+" nil 0 2)
                (goto-char (match-beginning 0)))))
        t))
     nil))
@@ -461,7 +501,8 @@ as it appears there."
 If this page is restricted and the task is linked to another
 page, remember to republish that page later and restrict the task
 as it appears there."
-  (let* ((link (save-match-data
+  (let* ((begin (match-end 0))
+         (link (save-match-data
                  (planner-task-link (planner-current-task-info))))
          (linked-page (if link (emacs-wiki-wiki-base link)))
          (linked-users
@@ -484,13 +525,19 @@ as it appears there."
     (if linked-page
         (planner-authz-republish-page-maybe linked-page))
 
-    ;; If the linked page has an authz restriction, restrict this note
+    ;; If the linked page has an authz restriction, restrict this task
 
     (when linked-users
-      (let ((start-tag
-             (format planner-authz-mason-content-tag-begin linked-users))
-            (end (point)))
-        (goto-char (+ 2 (match-end 0))) ; skip past "- "
+      (let* ((end (point))
+             (start-tag
+              (if (and planner-authz-appt-alt
+                       (save-match-data
+                         (goto-char begin)
+                         (re-search-forward planner-authz-appt-regexp end t)))
+                  (format planner-authz-mason-content-tag-alt-begin
+                          linked-users planner-authz-appt-alt)
+                (goto-char (+ 2 begin)) ; skip past "- "
+                (format planner-authz-mason-content-tag-begin linked-users))))
         (emacs-wiki-surround-text
          start-tag
          planner-authz-mason-content-tag-end
@@ -511,8 +558,7 @@ access will be shown."
     (message "Marking up index...")
     (goto-char (point-min))
     (while (re-search-forward "^- \\(.*\\)" nil t)
-      (message "match: %s" (match-string 1))
-      (let* ((match (match-string 1))
+      (let* ((match (emacs-wiki-wiki-base (match-string 1)))
              (users (if match (planner-authz-users match))))
         (if users
             (replace-match
