@@ -40,6 +40,8 @@
 ;; development version of planner.  This code may or may not work.  Use it at your
 ;; own risk.)
 ;;
+;;; Commentary:
+;; 
 ;; If you use Emacs' diary feature, planner-diary could be helpful for you.  It
 ;; puts all diary entries for the current day in the "* Diary" section of your day
 ;; plan page.  This section is updated every time you display the file in Emacs.
@@ -64,7 +66,7 @@
 ;; 1) You can put the following line of lisp code in your day plan pages to
 ;;    display your diary entries:
 ;;
-;;    <lisp>(planner-diary-entries-here)</lisp> 
+;;    <lisp>(planner-diary-entries-here)</lisp>
 ;;
 ;;    You can do this automatically for all day plan pages:
 ;;
@@ -119,6 +121,7 @@
 ;; - Bastien Guerry
 ;;   Bug report and appointment integration
 
+
 ;;; HISTORY:
 
 ;; version 1.0 First stable version.  Expected to work as documented.  Same
@@ -194,7 +197,9 @@
 (require 'planner)
 (require 'diary-lib)
 (require 'calendar)
+(require 'cl)
 
+;;; Code:
 (defgroup planner-diary nil
   "Diary integration for planner.el."
   :prefix "planner-diary-"
@@ -229,11 +234,15 @@ Uses the standard fancy-diary display."
   :group 'planner-diary)
 
 (defcustom planner-diary-number-of-days 1
-  "*Specifies how many days of diary entries are displayed in the standard
-diary section."
+  "*Number of days of diary entries displayed in the standard diary section."
   :type 'number
   :group 'planner-diary)
 
+(defcustom planner-diary-include-all-output-flag nil
+  "Non-nil means don't omit any data when copying diary entries into day pages."
+  :type 'boolean
+  :group 'planner-diary)
+
 ;; for backward compatability
 ;; (defcustom planner-diary-number-of-diary-entries 1
 ;;   "*Obsolete, use `planner-diary-number-of-days' instead."
@@ -243,13 +252,13 @@ diary section."
 
 ;; CAL-DESK
 (defcustom planner-diary-cal-desk-string "* Cal-Desk"
-  "*Header for the cal-desk section in a day plan page.
+  "Header for the cal-desk section in a day plan page.
 You might want to use \"* Diary\" or \"* Schedule\"."
   :type 'string
   :group 'planner-diary)
 
 (defcustom planner-diary-cal-desk-file planner-diary-file
-  "*The diary file you want to use for your cal-desk section."
+  "The diary file you want to use for your cal-desk section."
   :type 'file
   :group 'planner-diary)
 
@@ -257,15 +266,14 @@ You might want to use \"* Diary\" or \"* Schedule\"."
 ;;   (if (string-match planner-diary-cal-desk-string planner-day-page-template)
 ;;       t
 ;;     nil)
-  "*Non-nil means: Automatically insert a cal-desk diary section into day plan
-pages.  This section contains only entries from `planner-diary-cal-desk-file'."
+  "Non-nil means include a cal-desk section in day pages.
+This section contains only entries from `planner-diary-cal-desk-file'."
   :type 'boolean
   :group 'planner-diary)
 
 (defcustom planner-diary-cal-desk-number-of-days
   1                                     ; 1 is good for cal-desk's format.
-  "*Specifies how many days of diary entries are displayed in the cal-desk
-diary section."
+  "Number of days of diary entries displayed in the cal-desk diary section."
   :type 'number
   :group 'planner-diary)
 
@@ -292,15 +300,14 @@ to them."
 
 (defcustom planner-diary-appts-number-of-days
   1                                     ; 1 is good for appts
-  "*Specifies how many days of diary entries are displayed in the appointments
-diary section."
+  "Number of days of diary entries displayed in the appointments diary section."
   :type 'number
   :group 'planner-diary)
 
 (defcustom planner-diary-exclude-appts-from-diary
   planner-diary-use-appts            ; if you use appts, you want to exclude
                                      ; them from the diary section
-  "*Non-nil means that appts are not shown in the diary section.
+  "Non-nil means that appts are not shown in the diary section.
 This is useful if you use a diary appointments section.  Diary entries with a
 time assigned to them will then only be displayed in the diary appointments
 section and not in the diary section."
@@ -309,7 +316,7 @@ section and not in the diary section."
 
 ;; PRIVATE DIARY
 (defcustom planner-diary-private-string "* Private Diary"
-  "*Header for the private diary section in a day plan page.
+  "Header for the private diary section in a day plan page.
 This is the section you don't want to publish.  planner-diary doesn't do
 anything about the publishing, it's up to you to make sure this section doesn't
 get published."
@@ -318,7 +325,7 @@ get published."
 
 (defcustom planner-diary-private-file planner-diary-file
   ;; FIXME: Test if this works as buffer-local var
-  "*The diary file you want to use with for your private diary."
+  "The diary file you want to use with for your private diary."
   :type 'file
   :group 'planner-diary)
 
@@ -326,21 +333,20 @@ get published."
 ;;   (if (string-match planner-diary-private-string planner-day-page-template)
 ;;       t
 ;;     nil)
-  "*Non-nil means: Insert a private diary section into day plan pages.
+  "Non-nil means: Insert a private diary section into day plan pages.
 This section contains only entries from `planner-diary-private-file'."
   :type 'boolean
   :group 'planner-diary)
 
 (defcustom planner-diary-private-number-of-days
   planner-diary-number-of-days
-  "*Specifies how many days of diary entries are displayed in the private diary
-section."
+  "Number of days of diary entries displayed in the private diary section."
   :type 'number
   :group 'planner-diary)
 
 ;; PUBLIC DIARY
 (defcustom planner-diary-public-string "* Public Diary"
-  "*Header for the public diary section in a day plan page.
+  "Header for the public diary section in a day plan page.
 This is the section you want to publish.  Obviously, this only makes sense if
 you have a Private Diary section, too.  planner-diary doesn't do anything about
 the publishing, it's up to you to make sure this section gets published."
@@ -348,7 +354,7 @@ the publishing, it's up to you to make sure this section gets published."
   :group 'planner-diary)
 
 (defcustom planner-diary-public-file diary-file
-  "*The diary file you want to use with for your pbulic diary."
+  "The diary file you want to use with for your public diary."
   :type 'file
   :group 'planner-diary)
 
@@ -356,15 +362,14 @@ the publishing, it's up to you to make sure this section gets published."
 ;;   (if (string-match planner-diary-public-string planner-day-page-template)
 ;;       t
 ;;     nil)
-  "*Non-nil means: Insert a public diary section into day plan pages.
+  "Non-nil means: Insert a public diary section into day plan pages.
 This section contains only entries from `planner-diary-public-file'."
   :type 'boolean
   :group 'planner-diary)
 
 (defcustom planner-diary-public-number-of-days
   planner-diary-number-of-days
-  "*Specifies how many days of diary entries are displayed in the public diary
-section."
+  "Number of days of diary entries displayed in the public diary section."
   :type 'number
   :group 'planner-diary)
 
@@ -372,8 +377,7 @@ section."
 ;;; INTERNAL VARS
 ;; FIXME: Is this used for anything?
 (defcustom planner-diary-exclude-regexp ""
-  "*Regexp for diary entries that should not be displayed in the diary
-section.
+  "Regexp for diary entries not displayed in the diary section.
 Used by the schedule code."
   :type 'regexp
   :group 'planner-diary)
@@ -393,9 +397,10 @@ Used by the schedule code."
 (defun planner-diary-get-diary-entries (date &optional no-of-days file
                                              use-cal-desk)
   "Get the diary entries for DATE and the following NO-OF-DAYS days from FILE.
-DATE is a list (month day year).  NO-OF-DAYS defaults to
-`planner-diary-number-of-days'.  FILE defaults to
-`planner-diary-file'."
+DATE is a list (month day year). NO-OF-DAYS defaults to
+`planner-diary-number-of-days'. FILE defaults to
+`planner-diary-file'. Optional argument USE-CAL-DESK means
+display using a fancy desk calendar."
   (save-window-excursion
     (let* ((fancy-diary-buffer "temporary-fancy-diary-buffer")
            (entries)
@@ -407,7 +412,7 @@ DATE is a list (month day year).  NO-OF-DAYS defaults to
                                      fancy-schedule-display-desk-calendar)
               '(sort-diary-entries fancy-diary-display)))
            (no-of-days (or no-of-days planner-diary-number-of-days))
-           (diary-file (or  file planner-diary-file)))
+           (diary-file (or file planner-diary-file)))
       (list-diary-entries date no-of-days)
       (switch-to-buffer fancy-diary-buffer)
       (let ((inhibit-read-only t))
@@ -422,10 +427,12 @@ DATE is a list (month day year).  NO-OF-DAYS defaults to
                          (replace-match
                           (make-string (length (match-string 0)) ?-)))
                        (point-min))
-                   ;; remove date and lots of =s if just for 1 day
-                   (progn (goto-char (point-min))
-                          (search-forward-regexp "^=+$") ; one or more =
-                          (1+ (point))))
+                   (if planner-diary-include-all-output-flag
+                     ;; remove date and lots of =s if just for 1 day
+                       (point-min)
+                     (goto-char (point-min))
+                     (search-forward-regexp "^=+$") ; one or more =
+                     (1+ (point))))
                  ;; remove final newline
                  (progn
                    (goto-char (point-max))
@@ -434,10 +441,11 @@ DATE is a list (month day year).  NO-OF-DAYS defaults to
         (kill-buffer fancy-diary-buffer)
         entries))))
 
-(defun planner-diary-get-cal-desk-entries (date &optional no-of-days file) 
-  "Get the cal-desk styly diary entries for DATE and the following NO-OF-DAYS
-days from FILE.  DATE is a list (month day year).  NO-OF-DAYS defaults to
-`planner-diary-cal-desk-number-of-days'.  FILE defaults to
+(defun planner-diary-get-cal-desk-entries (date &optional no-of-days file)
+  "Get the cal-desk style diary entries for DATE.
+Consider the following NO-OF-DAYS days from FILE. DATE is a
+list (month day year). NO-OF-DAYS defaults to
+`planner-diary-cal-desk-number-of-days'. FILE defaults to
 `planner-diary-cal-desk-file'."
   (planner-diary-get-diary-entries date
                                    (or no-of-days
@@ -450,7 +458,8 @@ days from FILE.  DATE is a list (month day year).  NO-OF-DAYS defaults to
 ;;; TODO: Merge the following two functions into one:
 (defun planner-diary-get-appts-entries (date &optional no-of-days file)
   ;; FIXME: use optional args
-  "Call `planner-diary-get-entries' for appointments."
+  "Call `planner-diary-get-entries' for appointments on DATE.
+Arguments NO-OF-DAYS and FILE are ignored."
   (planner-diary-get-entries date 'appt))
 
 (defun planner-diary-get-entries (date &optional type)
@@ -461,8 +470,8 @@ DATE is a list (month day year).  Optional arg TYPE ..." ; FIXME: doc for type
 	   (no-of-days planner-diary-number-of-days)
 	   (init-entries (flet ((message (&rest args) (ignore args)))
 			   (list-diary-entries date (if type 1 no-of-days))))
-	   (end-entries 
-	    (mapcar 
+	   (end-entries
+	    (mapcar
 	     '(lambda (seq)
 		(let ((seq2 (if (and (eq type 'appt)
 				     (string-match planner-diary-time-regexp (cadr seq)))
@@ -473,7 +482,7 @@ DATE is a list (month day year).  Optional arg TYPE ..." ; FIXME: doc for type
 			       (string-match regexp seq2))
 					; then check appt not already there
 			  (string-match (substring seq2 0 (1- (length seq2)))
-					; bug in planner.el ? a space is added 
+					; bug in planner.el ? a space is added
 					; from diary entries to .diary.planner
 					(buffer-string))
 			  (if (eq type 'appt)
@@ -483,21 +492,21 @@ DATE is a list (month day year).  Optional arg TYPE ..." ; FIXME: doc for type
 				 (string-match planner-diary-time-regexp
 					       (cadr seq)))))
 		    seq2))) init-entries)))
-      (mapconcat 'eval (delete-duplicates (remove nil end-entries) 
+      (mapconcat 'eval (delete-duplicates (remove nil end-entries)
 					  :test 'equal) "\n"))))
 
 (defun planner-diary-appt-to-planner (appt)
-  "Convert appointments from diary format to planner format."
+  "Convert APPT from diary format to planner format."
   (string-match "\\([0-9]+:[0-9]+[ap]?m?\\)[ ]+\\(.*\\)" appt)
   (let ((time (match-string 1 appt))
 	(task (match-string 2 appt)))
     (concat time " | " task)))
 
 
-(defun planner-diary-get-private-entries (date &optional no-of-days file) 
-  "Get the private diary entries for DATE and the following NO-OF-DAYS days
-from FILE.  DATE is a list (month day year).  NO-OF-DAYS defaults to
-`planner-diary-private-number-of-days'.  FILE defaults to
+(defun planner-diary-get-private-entries (date &optional no-of-days file)
+  "Get private diary entries for DATE and the next NO-OF-DAYS from FILE.
+DATE is a list (month day year). NO-OF-DAYS defaults to
+`planner-diary-private-number-of-days'. FILE defaults to
 `planner-diary-private-file'."
   (planner-diary-get-diary-entries date
                                    (or no-of-days
@@ -505,10 +514,10 @@ from FILE.  DATE is a list (month day year).  NO-OF-DAYS defaults to
                                    (or file
                                        planner-diary-private-file)))
 
-(defun planner-diary-get-public-entries (date &optional no-of-days file) 
-  "Get the public diary entries for DATE and the following NO-OF-DAYS days
-from FILE.  DATE is a list (month day year).  NO-OF-DAYS defaults to
-`planner-diary-public-number-of-days'.  FILE defaults to
+(defun planner-diary-get-public-entries (date &optional no-of-days file)
+  "Get public diary entries for DATE and the next NO-OF-DAYS days from FILE.
+DATE is a list (month day year). NO-OF-DAYS defaults to
+`planner-diary-public-number-of-days'. FILE defaults to
 `planner-diary-public-file'."
   (planner-diary-get-diary-entries date
                                    (or no-of-days
@@ -523,8 +532,8 @@ from FILE.  DATE is a list (month day year).  NO-OF-DAYS defaults to
 ;;; LISP FUNCTIONS FOR USE IN PLANNER DAY PAGES
 ;;; arg FILE to specify a diary-file (suggested by David O'Toole)
 (defun planner-diary-entries-here (&optional file no-of-days)
-  "Display the diary entries from FILE for the next NO-OF-DAYS days.  FILE
-defaults to `planner-diary-file', NO-OF-DAYS defaults to
+  "Display the diary entries from FILE for the next NO-OF-DAYS days.
+FILE defaults to `planner-diary-file', NO-OF-DAYS defaults to
 `planner-diary-number-of-days'.
 
 Put this is your day pages:
@@ -540,8 +549,8 @@ You might want to use `planner-day-page-template' to do so."
    (or file planner-diary-file)))
 
 (defun planner-diary-appts-entries-here (&optional file no-of-days)
-  "Display the diary appointments entries from FILE for the next NO-OF-DAYS
-days.  FILE defaults to `planner-diary-appts-file', NO-OF-DAYS defaults to
+  "Display the diary appointments entries from FILE for the next NO-OF-DAYS.
+FILE defaults to `planner-diary-appts-file', NO-OF-DAYS defaults to
 `planner-diary-appts-number-of-days'.
 
 Put this is your day pages:
@@ -558,8 +567,8 @@ You might want to use `planner-day-page-template' to do so."
 
 
 (defun planner-diary-cal-desk-entries-here (&optional file no-of-days)
-  "Display the diary appointments entries from FILE for the next NO-OF-DAYS
-days.  FILE defaults to `planner-diary-cal-desk-file', NO-OF-DAYS defaults to
+  "Display the diary appointments entries from FILE for the next NO-OF-DAYS.
+FILE defaults to `planner-diary-cal-desk-file', NO-OF-DAYS defaults to
 `planner-diary-cal-desk-number-of-days'.
 
 Put this is your day pages:
@@ -575,8 +584,8 @@ You might want to use `planner-day-page-template' to do so."
    (or no-of-days planner-diary-cal-desk-number-of-days)))
 
 (defun planner-diary-public-entries-here (&optional file no-of-days)
-  "Display the diary appointments entries from FILE for the next NO-OF-DAYS
-days.  FILE defaults to `planner-diary-public-file', NO-OF-DAYS defaults to
+  "Display the diary appointments entries from FILE for the next NO-OF-DAYS.
+FILE defaults to `planner-diary-public-file', NO-OF-DAYS defaults to
 `planner-diary-public-number-of-days'.
 
 Put this is your day pages:
@@ -592,8 +601,8 @@ You might want to use `planner-day-page-template' to do so."
    (or no-of-days planner-diary-public-number-of-days)))
 
 (defun planner-diary-private-entries-here (&optional file no-of-days)
-  "Display the diary appointments entries from FILE for the next NO-OF-DAYS
-days.  FILE defaults to `planner-diary-private-file', NO-OF-DAYS defaults to
+  "Display the diary appointments entries from FILE for the next NO-OF-DAYS.
+FILE defaults to `planner-diary-private-file', NO-OF-DAYS defaults to
 `planner-diary-private-number-of-days'.
 
 Put this is your day pages:
@@ -612,7 +621,7 @@ You might want to use `planner-day-page-template' to do so."
 ;; There's a lot of code duplication in the following 3 functions.
 (defun planner-diary-update-section (file title text &optional force)
   ;;FIXME: Find a good place to insert a new section
-  "Update existing section TITLE in FILE by replacing existing text with TEXT.
+  "Update FILE's existing section TITLE by replacing existing text with TEXT.
 If optional arg FORCE is non-nil, update the section even if it doesn't exist,
 i.e. insert TITLE followed by TEXT at the top of the buffer."
   ;; search for something like "^* Diary$", delete buffer content to the next
@@ -644,7 +653,7 @@ i.e. insert TITLE followed by TEXT at the top of the buffer."
 
 
 (defun planner-diary-delete-section-text (file title)
-  "Delete the text of section TITLE in FILE."
+  "In FILE, delete the text of section TITLE."
   ;; search for something like "^* Diary$", delete buffer content to the next
   ;; "^* "
   ;; sanity checks
@@ -668,7 +677,7 @@ i.e. insert TITLE followed by TEXT at the top of the buffer."
 
 
 (defun planner-diary-delete-section (file title)
-  "Delete the whole section TITLE in FILE, text plus title."
+  "In FILE, delete the whole section TITLE, text plus title."
   ;; search for something like "^* Diary$", delete buffer content to the next
   ;; "^* "
   ;; sanity checks
@@ -928,62 +937,23 @@ C-cC-e updates the diary sections."
 
 (defalias 'planner-insinuate-diary 'planner-diary-insinuate)
 
-
-;;; DIARY ENTRIES FROM DAY PLAN PAGES
-;; part of the following code is stolen from make-diary-entry
-;; (defun planner-diary-(string &optional nonmarking file)
-;;   "Insert a diary entry STRING which may be NONMARKING in FILE.
-;; If omitted, NONMARKING defaults to nil and FILE defaults to diary-file."
-;;   (find-file-other-window
-;;    (substitute-in-file-name (if file file diary-file)))
-;;   (widen)
-;;   (goto-char (point-max))
-;;   (when (let ((case-fold-search t))
-;;           (search-backward "Local Variables:"
-;;                            (max (- (point-max) 3000) (point-min))
-;;                            t))
-;;     (beginning-of-line)
-;;     (insert "\n")
-;;     (previous-line 1))
-;;   (insert
-;;    (if (bolp) "" "\n")
-;;    (if nonmarking diary-nonmarking-symbol "")
-;;    string " "))
-
-;; From Jody Klymak
-;; planner-diary-insert-diary-entry
-;; planner-diary-find-diary-entry (switch to diary-file and put point on bol)
-;; planner-diary-delete-diary-entry
-;; planner-diary-edit-diary-entry
-
-
-;; ;; FIXME: add arg file
-;; ;; (defun planner-diary-guess-diary-file-from-buffer ())
-;; (defun planner-diary-add-entry (date text)
-;;   "Prompt for a diary entry to add to `diary-file'."
-;;   (interactive (list (planner-read-date)
-;;                      (read-string "Diary entry: ")))
-;; ;;   (save-excursion
-;; ;;     (set-buffer (find-file-noselect timelog-file))
-;;   (save-excursion
-;;     (save-window-excursion
-;;       (make-diary-entry
-;;        (concat
-;;         (let ((cal-date (planner-filename-to-calendar-date date)))
-;;           (calendar-date-string cal-date t t)
-;;           ;;           (if european-calendar-style
-;;           ;;               (format "%d/%d/%d"
-;;           ;;                       (elt cal-date 1)
-;;           ;;                       (elt cal-date 0)
-;;           ;;                       (elt cal-date 2))
-;;           ;;             (format "%d/%d/%d"
-;;           ;;                     (elt cal-date 0)
-;;           ;;                     (elt cal-date 1)
-;;           ;;                     (elt cal-date 2)))
-;;           )
-;;         " " text " "
-;;         (run-hook-with-args-until-success
-;;          'planner-annotation-functions))))))
-
+(defun planner-diary-add-entry (date time text)
+  "Prompt for a diary entry to add to `diary-file' on DATE.
+Uses `planner-annotation-functions' to make hyperlinks.
+TIME and TEXT are used in the description."
+  (interactive (list (planner-read-date)
+		     (read-string "Time: ")
+                     (read-string "Diary entry: ")))
+  (save-window-excursion
+    (make-diary-entry
+     (concat
+      (let ((cal-date (planner-filename-to-calendar-date date)))
+        (calendar-date-string cal-date t t))
+      " " time " " text " "
+      (run-hook-with-args-until-success
+       'planner-annotation-functions))
+     nil planner-diary-file)))
 
 (provide 'planner-diary)
+
+;;; planner-diary.el ends here
