@@ -3,8 +3,9 @@
 ;; Copyright (C) 2004  Free Software Foundation, Inc.
 
 ;; Author: Christophe Garion <garion@supaero.fr>
+;; Author: Sandra Jean Chua <sacha@free.net.ph>
 ;; Created: <2004-08-09 17:16:57 tof planner-mhe.el>
-;; Time-stamp: <2004-08-11 15:06:55 tof planner-mhe.el>
+;; Time-stamp: <2005-03-08 16:16:25 tof planner-mhe.el>
 ;; Keywords: planner, mh-e
 
 ;; This file is free software; you can redistribute it and/or modify
@@ -26,17 +27,66 @@
 
 ;; Inspired by planner-gnus (thanks to Sacha Chua)
 
-;; This file adds annotations for MH-E messages. You will then be
-;; able to use M-x planner-create-task-from-buffer to create tasks
-;; from MH-E folder or show buffers with the correct annotation.
+;; This file adds annotations for MH-E messages. You will then be able
+;; to use M-x planner-create-task-from-buffer to create tasks from
+;; MH-E folder or show buffers with the correct annotation. If you
+;; create an annotation from a mh-index folder, the message "real"
+;; folder will be taken into account.
 
 ;;; Code:
 
 (require 'planner)
 (require 'mh-e)
 
-(defun planner-mhe-get-message-id ()
-  "Return the ID of the current message."
+(defun planner-mhe-get-message-folder-from-index ()
+  "Returns the name of the message folder in a index folder
+  buffer."
+  (save-excursion
+    (mh-index-previous-folder)
+    (buffer-substring
+     (line-beginning-position)
+     (line-end-position))
+    )
+)
+
+(defun planner-mhe-get-message-real-folder ()
+  "Return the name of the current message real folder, so if you use
+  sequences, it will now work."  
+(save-excursion
+  (let* ((folder
+         (if (equal major-mode 'mh-folder-mode)
+             mh-current-folder
+           ;; Refer to the show buffer
+           mh-show-folder-buffer))
+        (end-index (min (length mh-index-folder) (length folder)))
+        )
+    ;; a simple test on mh-index-data does not work, because
+    ;; mh-index-data is always nil in a show buffer.
+    (if (string= mh-index-folder (substring folder 0 end-index))
+        (if (equal major-mode 'mh-show-mode)
+            (save-window-excursion
+              (when (buffer-live-p (get-buffer folder))
+                (progn
+                  (pop-to-buffer folder)
+                  (planner-mhe-get-message-folder-from-index)
+                  )
+              ))
+          (planner-mhe-get-message-folder-from-index)
+          )
+      folder
+      )
+    )))
+
+(defun planner-mhe-get-message-folder ()
+  "Return the name of the current message folder."  
+  (if (equal major-mode 'mh-folder-mode)
+      mh-current-folder
+    ;; Refer to the show buffer
+    mh-show-folder-buffer)
+  )
+
+(defun planner-mhe-get-message-field (field)
+  "Return a particular field of the current message."
   (save-excursion
     (let ((num
            (if (equal major-mode 'mh-folder-mode)
@@ -49,51 +99,22 @@
               (call-process (expand-file-name "anno" mh-progs)
                             nil t nil
                             folder
-                            "-list" "-component" "Message-ID"
+                            "-list" "-component" field
                             (number-to-string num))
               (buffer-string))
             "\n")))))
 
-(defun planner-mhe-get-message-num ()
-  "Return the number of the current message."
-  (save-excursion
-    (if (equal major-mode 'mh-folder-mode)
-        (number-to-string (mh-get-msg-num nil))
-      ;; Refer to the show buffer
-      (number-to-string (mh-show-buffer-message-number)))))
-
-(defun planner-mhe-get-message-folder ()
-  "Return the name of the current message folder. Be careful if you
-  use sequences."
-  (save-excursion
-    (if (equal major-mode 'mh-folder-mode)
-        mh-current-folder
-      ;; Refer to the show buffer
-      mh-show-folder-buffer)))
-
-(defun planner-mhe-get-header (header)
-  "Return a header of the message in folder mode. This will create a
-  show buffer for the corresponding message. If you have a more clever
-  idea..."
-  (let* ((folder mh-current-folder)
-         (num (mh-get-msg-num nil))
-         (buffer (get-buffer-create (concat "show-" folder))))
-  (with-current-buffer buffer
-    (mh-display-msg num folder)
-    (set-buffer buffer)
-    (mh-get-field header))))
-
 ;;;###autoload
 (defun planner-mhe-annotation ()
   "If called from a MH-E folder or message buffer, return an annotation.
 Suitable for use in `planner-annotation-functions'."
   (when (or (equal major-mode 'mh-folder-mode)
             (equal major-mode 'mh-show-mode))
-    (let ((from-header (planner-mhe-get-header "From:"))
-          (to-header (planner-mhe-get-header "To:")))
+    (let ((from-header (planner-mhe-get-message-field "From"))
+          (to-header (planner-mhe-get-message-field "To")))
       (planner-make-link
-       (concat "mhe://" (planner-mhe-get-message-folder) "/"
-               (planner-mhe-get-message-id))
+       (concat "mhe://" (planner-mhe-get-message-real-folder) "/"
+               (planner-mhe-get-message-field "Message-Id"))
        (concat "E-Mail "
                (if (and planner-ignored-from-addresses
                         from-header
@@ -113,22 +134,25 @@ Suitable for use in `planner-annotation-functions'."
     (let* ((folder (match-string 1 url))
 	   (num (match-string 2 url))
 	   (show-buf (concat "show-" folder)))
-      (get-buffer-create show-buf)
-      (mh-display-msg
-       (string-to-number
-        (if (= (aref num 0) ?<) ; message-id
-            (car (split-string 
-                  (with-temp-buffer
-                    (call-process
-                     (expand-file-name "pick" mh-progs)
-                     nil t nil
-                     folder
-                     "--message-id"
-                     (match-string 2 url))
-                    (buffer-string))
-                  "\n"))
-          num))
-       folder)
+      (save-window-excursion
+        (mh-visit-folder folder)
+        (get-buffer-create show-buf)
+        (mh-display-msg
+         (string-to-number
+          (if (= (aref num 0) ?<) ; message-id
+              (car (split-string 
+                    (with-temp-buffer
+                      (call-process
+                       (expand-file-name "pick" mh-progs)
+                       nil t nil
+                       folder
+                       "--message-id"
+                       num)
+                      (buffer-string))
+                    "\n"))
+            num))
+         folder)
+        )
       (pop-to-buffer show-buf))))
 
 ;(fset 'planner-get-from 'planner-gnus-get-address)
@@ -138,5 +162,8 @@ Suitable for use in `planner-annotation-functions'."
 (add-hook 'planner-annotation-functions 'planner-mhe-annotation)
 (add-to-list 'emacs-wiki-url-protocols '("mhe" planner-mhe-browse-url nil))
 
+; to set the mh-path etc. variables
+(mh-find-path)
+
 (provide 'planner-mhe)
 ;;; planner-mhe.el ends here
