@@ -29,9 +29,10 @@
 ;;
 
 (require 'planner)
-(condition-case nil
+(condition-case err
     (require 'crm)
-  (error (message "Could not load crm; no completing read available")))
+  ('file-error
+   (message "Could not load crm; cumbersome completing read used")))
 
 ;;; Code:
 
@@ -126,8 +127,8 @@ separately."
   (if (and (nth 5 info) (listp (nth 5 info)))
       (let ((current (nth 5 info)))
         (while current
-          (when (string-match planner-date-regexp (car current))
-            (setq ad-return-value (car current))
+          (when (string-match planner-date-regexp (emacs-wiki-wiki-base (car current)))
+            (setq ad-return-value (emacs-wiki-wiki-base (car current)))
             (setq current nil))
           (setq current (cdr current))))
     ad-do-it))
@@ -211,7 +212,7 @@ separately."
                       (delete-region (line-beginning-position)
                                      (min (point-max) (1+ (line-end-position)))))
                     ;; Update
-                    (setq links (delete (planner-task-date info) links))
+                    (setq links (planner-multi-link-delete (planner-task-date info) links))
                     (planner-find-file (planner-link-base (car links)))
                     (when date (setq links (cons date links)))
                     (if (planner-find-task info)
@@ -229,6 +230,74 @@ separately."
       ad-do-it)
       (when (planner-replan-task date) t))))
 
+(setq planner-replan-task-function 'planner-multi-replan-task)
+
+(defun planner-multi-replan-task (page-name)
+  "Change or assign the plan page for the current task.
+PAGE-NAME is the new plan page for the task. Use
+`planner-copy-or-move-task' if you want to change the date.
+With a prefix, provide the current link text for editing."
+  (let ((info (planner-current-task-info))
+	old-pages ; Pages the task is currently assigned to
+	new-pages
+	new-task
+	date) ; Pages from page-name
+    (if (and 
+	 ;; The proposed link is not a multilink
+	 (or (not page-name)
+	     (not (string-match planner-multi-separator page-name)))
+	 ;; The current link is not a multilink
+	 (not (and (planner-task-link-text info)
+		   (string-match planner-multi-separator 
+				 (planner-task-link-text info)))))
+	(planner-replan-task-basic page-name)
+      (with-planner-update-setup
+       ;;; Do the funky multi magic here
+       (setq old-pages (planner-multi-task-link-as-list info))
+       ;; Adjust the past page name if we're coming from a task with a date
+       (setq date (planner-task-date info))
+       (when (and date
+		  (not (planner-multi-link-member date old-pages)))
+	 (setq old-pages (cons date old-pages)))
+       ;; Set up the new links list for easy testing
+       (setq new-pages (mapcar 'emacs-wiki-wiki-base (planner-multi-split page-name)))
+        ;; Adjust the future page name if we're coming from a task with a date
+       (unless (member date new-pages)
+	 (setq page-name 
+	       (concat date planner-multi-separator page-name))
+	 (setq new-pages (cons date new-pages)))
+       ;; Remove date if it's just that
+       (when (equal new-pages (list date))
+	 (setq page-name nil)
+	 (setq new-pages nil))
+       (setq new-task (planner-format-task info nil nil nil nil
+					   (or (planner-make-link page-name) "")
+					   (or page-name "")))
+       ;; Map over the old pages, removing if not in the new one and updating if it is
+       ;; We use mapcar instead of mapc for emacs20 compatibility
+       (mapcar (lambda (page)
+                 (planner-find-file page)
+                 (cond
+                  ((planner-find-task info)
+                   (delete-region (line-beginning-position)
+                                  (1+ (line-end-position))))
+                  ((member (planner-page-name) new-pages)
+                   (planner-seek-task-creation-point)))
+                 (when (or (string-match planner-date-regexp (planner-page-name))
+                           (member (planner-page-name) new-pages))
+                   (insert new-task "\n"))
+                 (setq new-pages (delete (planner-page-name) new-pages)))
+               old-pages)
+       ;; Map over any new pages that were not included
+       (mapcar (lambda (page)
+                 (planner-find-file page)
+                 (if (planner-find-task info)
+                     (delete-region (line-beginning-position)
+                                    (1+ (line-end-position)))
+                   (planner-seek-task-creation-point))
+                 (insert new-task "\n"))
+               new-pages)))))
+
 (defadvice planner-update-task (around planner-multi activate)
   "Copy the text from this task to the linked tasks, if any."
   (let ((info (planner-current-task-info)))
@@ -250,6 +319,28 @@ separately."
 	       (insert (planner-format-task info) "\n"))
 	     (setq links (cdr links))))))))
 
+(defun planner-multi-tasks-equal-p (task-a task-b)
+  "Return t if TASK-A and TASK-B are equivalent.
+This is true if they have the same value for priority, status,
+description, and links."
+  (and (string= (or (planner-task-priority task-a) "")
+                (or (planner-task-priority task-b) ""))
+       (string= (or (planner-task-status task-a) "")
+                (or (planner-task-status task-b) ""))
+       (string= (or (planner-task-description task-a) "")
+                (or (planner-task-description task-b) ""))
+       (string= (or (planner-task-link task-a) "")
+                (or (planner-task-link task-b) ""))
+       (if (or (string-match (regexp-quote planner-multi-separator) (planner-task-link-text task-a))
+               (string-match (regexp-quote planner-multi-separator) (planner-task-link-text task-b)))
+           (string= (planner-task-link-text task-a)
+                    (planner-task-link-text task-b))
+         (and (string= (or (planner-task-plan task-a) "")
+                       (or (planner-task-plan task-b) ""))
+              (string= (or (planner-task-date task-a) "")
+                       (or (planner-task-date task-b) ""))))))
+(defalias 'planner-tasks-equal-p 'planner-multi-tasks-equal-p)
+
 (defadvice planner-edit-task-description (around planner-multi activate)
   "Copy the text from this task to the linked tasks, if any."
   (let* ((info (planner-current-task-info))
@@ -288,18 +379,30 @@ separately."
 
 (defadvice planner-make-link (around planner-multi activate)
   "Escape separately if using `planner-multi-separator'."
-  (let ((case-fold-search nil))
-    (cond
-     ((listp link)
-      (setq ad-return-value (mapconcat 'emacs-wiki-make-link
-                                       link
-                                       planner-multi-separator)))
-     ((string-match (regexp-quote planner-multi-separator) link)
-      (setq ad-return-value (mapconcat 'emacs-wiki-make-link
-                                       (planner-multi-split link)
-                                       planner-multi-separator)))
-     (t ad-do-it))))
-      
+  (if (ad-get-arg 2)
+      ad-do-it
+    (let ((case-fold-search nil))
+      (cond
+       ((listp link)
+        (setq ad-return-value (mapconcat 'emacs-wiki-make-link
+                                         link
+                                         planner-multi-separator)))
+       ((string-match (regexp-quote planner-multi-separator) link)
+        (setq ad-return-value (mapconcat 'emacs-wiki-make-link
+                                         (planner-multi-split link)
+                                         planner-multi-separator)))
+       (t ad-do-it)))))
+
+(defun planner-multi-filter-links (regexp links-list &optional nonmatch)
+  "Return a list of links matching REGEXP in LINKS-LIST.
+If NONMATCH is non-nil, return non-matching links instead."
+  (delq nil
+        (mapcar (lambda (item)
+                  (when (funcall (if nonmatch 'not 'null)
+                                 (string-match regexp item))
+                    item))
+                links-list)))
+
 (defadvice planner-create-task-from-info (around planner-multi activate)
   "Create task on multiple pages if necessary."
   (setq link-text (or link-text (planner-task-link-text info)))
@@ -310,10 +413,12 @@ separately."
         (let ((list (planner-multi-split (or link-text (planner-task-link-text info))))
               (link-text (or link-text (planner-task-link-text info))))
           (when (or date (planner-task-date info))
+            (setq list (planner-multi-filter-links
+                        planner-date-regexp
+                        list
+                        t))
             (add-to-list 'list (or date (planner-task-date info)))
-            (setq link-text (concat (or date (planner-task-date info))
-                                    planner-multi-separator
-                                    link-text)))
+            (setq link-text (planner-multi-make-link list)))
           (save-window-excursion
             (while list
               (planner-find-file (planner-link-base (car list)))
@@ -493,21 +598,31 @@ separately."
           (let* ((info (planner-current-note-info))
                  (links (planner-multi-note-link-as-list info))
                  (new-links (planner-multi-split page))
-                 (cursor new-links)
+		 (old-anchor (planner-make-link 
+			      (concat (planner-note-page info) "#"
+				      (planner-note-anchor info))))
+		 cursor
+                 current-page
                  (old-pages (mapcar 'emacs-wiki-wiki-base links)))
+	    (when (and (not old-pages)
+		       (string-match planner-date-regexp (planner-note-page info)))
+	      (setq old-pages (list (planner-note-page info)))
+              (add-to-list 'new-links old-anchor))
             ;; Add to new pages
+	    (setq cursor new-links)
             (while cursor
-              (when (not (member (emacs-wiki-wiki-base (car cursor)) old-pages))
+              (setq current-page (planner-link-base (car cursor)))
+              (when (not (member current-page old-pages))
                 (setcar cursor
                         (planner-make-link
                          (format "%s#%d"
-                                 (car cursor)
-                                 (planner-create-note (emacs-wiki-wiki-base (car cursor)))))))
+                                 (planner-link-base (car cursor))
+                                 (planner-create-note current-page)))))
               (setq cursor (cdr cursor)))
             ;; Delete from pages removed from list
             (setq cursor links)
             (while cursor
-              (unless (member (emacs-wiki-wiki-base (car cursor)) old-pages)
+              (unless (member (planner-link-base (car cursor)) old-pages)
                 (emacs-wiki-visit-link (emacs-wiki-wiki-link-target (car links)))
                 (save-restriction
                   (when (planner-narrow-to-note)
@@ -582,14 +697,20 @@ separately."
 (defun planner-multi-read-name (file-alist &optional prompt initial)
   "Read multiple pages, completing based on FILE-ALIST.
 If PROMPT is specified, use that instead of \"Page:\"."
-  (let* ((map (copy-keymap minibuffer-local-completion-map))
+  (let* ((minibuffer-prompt-properties
+	  (plist-put 
+	   (copy-sequence minibuffer-prompt-properties)
+	   'read-only nil))
+	 (map (copy-keymap minibuffer-local-completion-map))
          (completion-ignore-case t)
          (crm-separator (regexp-quote planner-multi-separator))
-         (prompt (format "%s(default: %s) " (or prompt "Page: ") planner-default-page))
+         (prompt (format "%s(default: %s) " 
+			 (or prompt "Page: ") planner-default-page))
          str)
     (unwind-protect
         (progn
-          (define-key minibuffer-local-completion-map planner-multi-separator 'self-insert-command)
+          (define-key minibuffer-local-completion-map 
+	    planner-multi-separator 'self-insert-command)
           (setq str
                 (if (fboundp 'completing-read-multiple)
                     (completing-read-multiple
@@ -597,7 +718,8 @@ If PROMPT is specified, use that instead of \"Page:\"."
                      'planner-history-list
                      planner-default-page)
                   (planner-multi-split
-                   (read-string prompt initial 'planner-history-list planner-default-page))))
+                   (read-string prompt initial 'planner-history-list
+				planner-default-page))))
           (cond
            ((or (null str)
                 (string= (car str) "")) planner-default-page)
@@ -605,11 +727,28 @@ If PROMPT is specified, use that instead of \"Page:\"."
            (t (mapconcat 'emacs-wiki-make-link str planner-multi-separator))))
       (setq minibuffer-local-completion-map map))))
 
-(defadvice planner-read-name (around planner-multi activate)
-  "Read multiple names."
-  (if (fboundp 'completing-read-multiple)
-      (setq ad-return-value (planner-multi-read-name file-alist prompt initial))
-    ad-do-it))
+(defun planner-multi-read-name-multiple-prompts (file-alist prompt initial)
+  "Read multiple pages, completing based on FILE-ALIST.
+If PROMPT is specified, use that instead of \"Page:\".
+Enter one plan page at a time. To end input, type \"nil\"."
+  (let ((more t)
+        input
+        last
+        list)
+    (while more
+      (setq input
+	    (planner-read-name-single file-alist
+				      (concat "nil to stop. " (or prompt "Page: "))
+				      initial))
+      (setq planner-default-page nil initial nil)
+      (if input
+	  (add-to-list 'list input t)
+	(setq more nil)))
+    (setq planner-default-page (planner-multi-make-link list))))
+
+(if (fboundp 'completing-read-multiple)
+    (setq planner-read-name-function 'planner-multi-read-name)
+  (setq planner-read-name-function 'planner-multi-read-name-multiple-prompts))
 
 (provide 'planner-multi)
 
