@@ -102,6 +102,13 @@
   :type 'string
   :group 'planner-timeclock-summary)
 
+(defcustom planner-timeclock-summary-placeholder-char "."
+  "Placeholder for blank cells in the report table.
+If there are have blank cells in a simple table, the generated HTML
+table will mess up. This character will be used as a placeholder."
+  :type 'character
+  :group 'planner-timeclock-summary)
+
 (defcustom planner-timeclock-summary-include-sub-plan-pages-flag nil
   "Non-nil means include 'sub plan pages' when doing plan page reports.
 
@@ -177,7 +184,7 @@ project field in `timeclock-log-data' is 'project: task' here."
                   (setq project-name planner-timeclock-summary-not-planned-string)
                   (setq task-name task-fullname))
               (setq task-name (replace-in-string
-                               task-fullname "^.*?: " "" t))))
+                               task-fullname "^\\(.*?\\): " "" t))))
         ;; no ": " found, not planned task
         (setq task-name task-fullname)))
     (list project-name task-name task-length)))
@@ -454,9 +461,12 @@ not include the summary."
             (goto-char (point-min))
             (while (search-forward
                     planner-timeclock-summary-empty-cell-string nil t)
-              (replace-match (make-string
-                              (length planner-timeclock-summary-empty-cell-string)
-                              (aref " " 0))))
+              (replace-match (concat
+			      planner-timeclock-summary-placeholder-char
+			      (make-string
+			       (- (length planner-timeclock-summary-empty-cell-string) 
+				  1)
+                              (aref " " 0)))))
             (goto-char (point-max))
             (unless hide-summary
               (insert (planner-timeclock-summary-make-summary-string-range
